#
# copyObject, pasteObject, cutObject, frontObject, backObject
# -- self-explaining routines to manipulate object selections.
#
proc copyObject {} {
	global sld sldi sldimg sldfnt cv
	set sld(cutbuffer) {}
	foreach i [$cv find withtag $sld(obj)] {
		set cutbufitem ""
		if {[$cv type $i] == "text"} {
			 set cutbufitem "set sldfnt($i) $sldfnt($i);"
		} elseif {[$cv type $i] == "image"} {
			 set cutbufitem "set sldimg($i) $sldimg($i);"
		}
		append cutbufitem [describeObj $i notag]
		lappend sld(cutbuffer) $cutbufitem
	}
puts "cutbuffer: $sld(cutbuffer)"
}

proc pasteObject {} {
	global sld sldi cv
	set sld(page,update) 1
	set tag $sld(pastecnt)
	incr sld(pastecnt)
	foreach cmd $sld(cutbuffer) {
		append cmd " -tags \{slide gp$tag\}"
#puts "paste: $cmd"
		eval $cmd
	}
}

proc cutObject {} {
	global sld sldi cv
	set sld(page,update) 1
	copyObject
	catch {$cv delete $sld(obj)}
}

proc frontObject {} {
	global sld sldi cv
	set sld(page,update) 1
	$cv raise $sld(obj)
}

proc backObject {} {
	global sld sldi cv
	set sld(page,update) 1
	$cv lower $sld(obj)
	$cv lower background
}

## terminate objectOptions (see below)
proc applyObjectOptions { i opts } {
	global cv
	foreach op $opts {
		$cv itemconfig $i -$op [.opt.e$op get]
	}
	destroy .opt
}

#
# objectOptions -- dialog to change some object options, according to
# the list in "foreach op {anchor ...}".  Only options found for the object
# are shown.
#
proc objectOptions { x y } {
	global cv
	set sld(page,update) 1
	toplevel .opt
	wm transient .opt .	
	set x [$cv canvasx $x]
	set y [$cv canvasy $y]
	set i [$cv find closest $x $y]
	set dbopt {}
	foreach j [$cv itemconfig $i] {
		lappend dbopt [lindex $j 0]
	}
	set opts {}
	set row 0
	foreach op {anchor fill outline width font justify text} {
		if {[lsearch $dbopt -$op] >= 0} {
			label .opt.l$op -text $op -highlightthickness 0
			entry .opt.e$op
			.opt.e$op insert 0 [lindex [$cv itemconfig $i -$op] 4]
			grid .opt.l$op -row $row -column 0 -padx 4 -pady 4
			grid .opt.e$op -row $row -column 1
			lappend opts $op
			incr row
		}
	} 
	frame .opt.bts
	button .opt.bts.cancel -text cancel -command {destroy .opt}
	button .opt.bts.apply -text apply -command "applyObjectOptions \
		$i [list $opts]"
	pack .opt.bts.cancel .opt.bts.apply -side left -fill x -expand 1
	grid .opt.bts -row $row -columnspan 2 -padx 10 -pady 10
	centerWindow .opt
	raise .opt
	focus .opt
	grab .opt
}


#
# describeObj -- formats a command in tcl for recreating the object given,
# so we may save each object as a command to create it.
# Example: a line will return a string like this,
#   ".c.c create line 180 125 195 200 -fill blue -width 4", where the
# numbers are line coordinates and "blue","4" are the object properties.
# (see "man canvas" to get all the details :)
#
proc describeObj {i {notag {}}} {
	global sldi sld sldimg sldfnt cv
### the following "foreach" replaces this
### taking into account	scale factor --> set xy [join [$cv coords $i]]
	set xy ""
	foreach x [$cv coords $i] {
		append xy "[expr $x/$sld(scale)] "
	}
	if {[$cv type $i] == "image"} {
			if {[info exists sldimg($i)]} {
				set img $sldimg($i)
			} else {
				set img [$cv itemcget $i -image]
			}
			set dsc "set sldi($img) \"$sldi($img)\"
image create photo $img -data \$sldi($img)
\$cv create image $xy -anchor c -image $img"
	} elseif {[$cv type $i] == "text"} {
			if {[info exists sldfnt($i)]} {
				set fnt $sldfnt($i)
			} else {
				set fnt [$cv itemcget $i -font]
			}
		set dsc "\$cv create text $xy"
		foreach cf [$cv itemconfig $i] {
			if {[lindex $cf 0] == "-font"} {
				append dsc " -font \{$fnt\}"
			} elseif {([lindex $cf 3] != [lindex $cf 4]) && \
				([lindex $cf 0] != "-tags") } {
					append dsc " [lindex $cf 0] \{[lindex $cf 4]\}"
			}
		}
	} else {
		set dsc "\$cv create [.c.c type $i] $xy"
		foreach cf [$cv itemconfig $i] {
			if {([lindex $cf 3] != [lindex $cf 4]) && \
				([lindex $cf 0] != "-tags") } {
					append dsc " [lindex $cf 0] \{[lindex $cf 4]\}"
			}
		}
	}
## now append all tags belonging to this object
	if {$notag != "notag"} {
		append dsc " -tags \{[$cv gettags $i]\}"
	}
	set dsc
}
